<div>This view shows a controller which uses a polling mechanism to
  contact the server. It is constantly using angular's $timeout.</div>
<button id="pollstarter" ng-click="startPolling()">Start Polling</button>
<div>{{count}}</div>
