<span>Outer: {{item.reusedBinding}}</span>
<span>Other other: {{item.alsoReused}}</span>

<div id="baz">
  <div ng-repeat="item in wrapper">
    <span>Inner: {{item.reusedBinding}}</span>
    <span>Inner other: {{item.alsoReused}}</span>
  </div>
</div>
