<p>A series of repeaters used in tests.</p>
<ul class="allinfo"><li ng-repeat="allinfo in days">
  <span>{{allinfo.initial}}</span>
  <span>{{allinfo.name}}</span>
</li></ul>
<ul><li ng-repeat="baz in days | filter:'T'">
  <span>{{baz.initial}}</span>
</li></ul>
<ul><li ng-repeat="baz in tDays = (days | filter:'T')">
  <span>{{baz.initial}}</span>
</li></ul>
<ul><li data-ng-repeat="day in days">
  <span>{{day.initial}}</span>
</li></ul>
<ul><li ng:repeat="bar in days">
  <span>{{bar.initial}}</span>
</li></ul>
<div ng-repeat-start="bloop in days">
  <span>{{bloop.initial}}</span>
</div>
<span>-</span>
<div ng-repeat-end>
  <span>{{bloop.name}}</span>
</div>
