/* app css stylesheet */

.menu {
  list-style: none;
  border-bottom: 0.1em solid black;
  margin-bottom: 2em;
  padding: 0 0 0.5em;
}

.menu:before {
  content: "[";
}

.menu:after {
  content: "]";
}

.menu > li {
  display: inline;
}

.menu > li:before {
  content: "|";
  padding-right: 0.3em;
}

.menu > li:nth-child(1):before {
  content: "";
  padding: 0;
}

.ng-scope {
  background-color: rgba(0,0,0,.05);
}

.ng-binding {
  border: 1px solid rgba(50, 200, 50, .8);
}

#chat-box {
  width: 300px;
  height: 200px;
  padding: 25px;
  border: 2px solid;
  margin: 25px;
  overflow: scroll;
}
