#animationTop .animate-if {
  background:white;
  border:1px solid black;
  padding:10px;
}

#animationTop .animate-if.ng-enter,
#animationTop .animate-if.ng-leave {
  -webkit-transition:all linear 1s;
  -moz-transition:all linear 1s;
  -o-transition:all linear 1s;
  transition:all linear 1s;

}

#animationTop .animate-if.ng-enter,
#animationTop .animate-if.ng-leave.ng-leave-active {
  opacity:0;
}

#animationTop .animate-if.ng-leave,
#animationTop .animate-if.ng-enter.ng-enter-active {
  opacity:1;
}
